@@ -138,7 +138,8 @@ typedef struct {
 		bool xdp_tcp;
 		bool xdp_route_check;
 		int ctl_timeout;
-		conf_val_t srv_nsid;
+		const uint8_t *srv_nsid_data;
+		size_t srv_nsid_len;
 		bool srv_ecs;
 		bool srv_ans_rotate;
 	} cache;
@@ -193,15 +194,6 @@ int conf_refresh_txn(
 	conf_t *conf
 );
 
-/*!
- * Refreshes cached hostname.
- *
- * \param[in] conf  Configuration.
- */
-void conf_refresh_hostname(
-	conf_t *conf
-);
-
 /*!
  * Creates new or opens old configuration database.
  *
