@@ -24,18 +24,20 @@
 #include "contrib/time.h"
 #include "knot/include/module.h"
 
-#define MOD_SINK	"\x04""sink"
-#define MOD_IDENTITY	"\x08""identity"
-#define MOD_VERSION	"\x07""version"
-#define MOD_QUERIES	"\x0B""log-queries"
-#define MOD_RESPONSES	"\x0D""log-responses"
+#define MOD_SINK		"\x04""sink"
+#define MOD_IDENTITY		"\x08""identity"
+#define MOD_VERSION		"\x07""version"
+#define MOD_QUERIES		"\x0B""log-queries"
+#define MOD_RESPONSES		"\x0D""log-responses"
+#define MOD_WITH_QUERIES	"\x16""responses-with-queries"
 
 const yp_item_t dnstap_conf[] = {
-	{ MOD_SINK,      YP_TSTR,  YP_VNONE },
-	{ MOD_IDENTITY,  YP_TSTR,  YP_VNONE },
-	{ MOD_VERSION,   YP_TSTR,  YP_VNONE },
-	{ MOD_QUERIES,   YP_TBOOL, YP_VBOOL = { true } },
-	{ MOD_RESPONSES, YP_TBOOL, YP_VBOOL = { true } },
+	{ MOD_SINK,         YP_TSTR,  YP_VNONE },
+	{ MOD_IDENTITY,     YP_TSTR,  YP_VNONE },
+	{ MOD_VERSION,      YP_TSTR,  YP_VNONE },
+	{ MOD_QUERIES,      YP_TBOOL, YP_VBOOL = { true } },
+	{ MOD_RESPONSES,    YP_TBOOL, YP_VBOOL = { true } },
+	{ MOD_WITH_QUERIES, YP_TBOOL, YP_VBOOL = { false } },
 	{ NULL }
 };
 
@@ -56,8 +58,33 @@ typedef struct {
 	size_t identity_len;
 	char *version;
 	size_t version_len;
+	bool with_queries;
 } dnstap_ctx_t;
 
+static void msg_query_qname_restore(Dnstap__Message *msg, knotd_qdata_t *qdata)
+{
+	if (msg->query_message.data == NULL) {
+		return;
+	}
+
+	const knot_dname_t *orig_qname = knotd_qdata_orig_qname(qdata);
+	if (orig_qname == NULL) {
+		return;
+	}
+
+	memcpy(msg->query_message.data + KNOT_WIRE_HEADER_SIZE,
+	       orig_qname, qdata->query->qname_size);
+}
+
+static void msg_query_qname_case_lower(Dnstap__Message *msg)
+{
+	if (msg->query_message.data == NULL) {
+		return;
+	}
+
+	knot_dname_to_lower(msg->query_message.data + KNOT_WIRE_HEADER_SIZE);
+}
+
 static knotd_state_t log_message(knotd_state_t state, const knot_pkt_t *pkt,
                                  knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
@@ -116,10 +143,22 @@ static knotd_state_t log_message(knotd_state_t state, const knot_pkt_t *pkt,
 		dnstap.has_version = 1;
 	}
 
+	/* Also add query message if 'responses-with-queries' is enabled and this is a response. */
+	if (ctx->with_queries &&
+	    msgtype == DNSTAP__MESSAGE__TYPE__AUTH_RESPONSE &&
+	    qdata->query != NULL)
+	{
+		msg.query_message.len = qdata->query->size;
+		msg.query_message.data = qdata->query->wire;
+		msg.has_query_message = 1;
+	}
+
 	/* Pack the message. */
 	uint8_t *frame = NULL;
 	size_t size = 0;
+	msg_query_qname_restore(&msg, qdata);
 	dt_pack(&dnstap, &frame, &size);
+	msg_query_qname_case_lower(&msg);
 	if (frame == NULL) {
 		return state;
 	}
@@ -249,6 +288,10 @@ int dnstap_load(knotd_mod_t *mod)
 	}
 	ctx->version_len = (ctx->version != NULL) ? strlen(ctx->version) : 0;
 
+	/* Set responses-with-queries. */
+	conf = knotd_conf_mod(mod, MOD_WITH_QUERIES);
+	ctx->with_queries = conf.single.boolean;
+
 	/* Set sink. */
 	conf = knotd_conf_mod(mod, MOD_SINK);
 	const char *sink = conf.single.string;
