@@ -63,6 +63,13 @@ static int ixfr_put_chg_part(knot_pkt_t *pkt, struct ixfr_proc *ixfr,
 		    knot_soa_serial(ixfr->cur_rr.rrs.rdata) == ixfr->soa_to) {
 			break;
 		}
+
+		if (pkt->size > KNOT_WIRE_PTR_MAX) {
+			// optimization: once the XFR DNS message is > 16 KiB, compression
+			// is limited. Better wrap to next message.
+			return KNOT_ESPACE;
+		}
+
 		IXFR_SAFE_PUT(pkt, &ixfr->cur_rr);
 		if (ixfr->cur_rr.type == KNOT_RRTYPE_SOA) {
 			ixfr->in_remove_section = !ixfr->in_remove_section;
@@ -125,7 +132,7 @@ static int ixfr_query_check(knotd_qdata_t *qdata)
 {
 	NS_NEED_ZONE(qdata, KNOT_RCODE_NOTAUTH);
 	NS_NEED_AUTH(qdata, ACL_ACTION_TRANSFER);
-	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL);
+	NS_NEED_ZONE_CONTENTS(qdata);
 
 	/* Need SOA authority record. */
 	const knot_pktsection_t *authority = knot_pkt_section(qdata->query, KNOT_AUTHORITY);
@@ -276,6 +283,9 @@ int ixfr_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 			            knot_strerror(ret));
 			return KNOT_STATE_FAIL;
 		}
+	} else if (qdata->params->xdp_msg != NULL) {
+		qdata->rcode = KNOT_RCODE_SERVFAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Reserve space for TSIG. */
