@@ -855,9 +855,6 @@ static int reload_conf(conf_t *new_conf)
 		log_error("failed to migrate configuration (%s)", knot_strerror(ret));
 	}
 
-	/* Refresh hostname. */
-	conf_refresh_hostname(new_conf);
-
 	return KNOT_EOK;
 }
 
@@ -957,7 +954,7 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 		warn_bg = false;
 	}
 
-	if (warn_listen && listen_changed(conf, server)) {
+	if (warn_listen && server->ifaces != NULL && listen_changed(conf, server)) {
 		log_warning(msg, "listen(-xdp)");
 		warn_listen = false;
 	}
