@@ -719,16 +719,20 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 
 	zone_set_flag(update->zone, ZONE_IS_CATALOG);
 
-	int ret = KNOT_EOK;
+	int ret = catalog_zone_verify(update->new_cont);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	ssize_t upd_count = 0;
 	if ((update->flags & UPDATE_INCREMENTAL)) {
 		ret = catalog_update_from_zone(update->zone->catalog_upd,
 		                               update->change.remove, update->new_cont,
-		                               true, false, update->zone->catalog, &upd_count);
+		                               true, update->zone->catalog, &upd_count);
 		if (ret == KNOT_EOK) {
 			ret = catalog_update_from_zone(update->zone->catalog_upd,
 			                               update->change.add, update->new_cont,
-			                               false, false, NULL, &upd_count);
+			                               false, NULL, &upd_count);
 		}
 	} else {
 		ret = catalog_update_del_all(update->zone->catalog_upd,
@@ -737,7 +741,7 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 		if (ret == KNOT_EOK) {
 			ret = catalog_update_from_zone(update->zone->catalog_upd,
 			                               update->new_cont, update->new_cont,
-			                               false, true, NULL, &upd_count);
+			                               false, NULL, &upd_count);
 		}
 	}
 
@@ -746,6 +750,11 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 		if (kill(getpid(), SIGUSR1) != 0) {
 			ret = knot_map_errno();
 		}
+	} else {
+		// this cant normally happen, just some ENOMEM or so
+		(void)catalog_update_del_all(update->zone->catalog_upd,
+		                             update->zone->catalog,
+		                             update->zone->name, &upd_count);
 	}
 
 	return ret;
@@ -882,24 +891,6 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 
 	conf_val_t val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
 	bool dnssec = conf_bool(&val);
-	val = conf_zone_get(conf, C_ZONEMD_GENERATE, update->zone->name);
-	unsigned digest_alg = conf_opt(&val);
-	bool do_digest = (digest_alg != ZONE_DIGEST_NONE && !dnssec); // in case of DNSSEC, digest is part of signing routine
-	if (do_digest && !(update->flags & UPDATE_FULL) && zone_update_to(update) == NULL) {
-		// cold start, decide if (digest & bump SOA) or NOOP
-		// yes, computing hash twice, but in rare situation: cold start & exists & invalid
-		if (zone_contents_digest_exists(update->new_cont, digest_alg, false)) {
-			do_digest = false;
-		} else {
-			ret = zone_update_increment_soa(update, conf);
-		}
-	}
-	if (do_digest && ret == KNOT_EOK) {
-		ret = zone_update_add_digest(update, digest_alg, false);
-	}
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
 
 	conf_val_t thr = conf_zone_get(conf, C_ADJUST_THR, update->zone->name);
 	if ((update->flags & (UPDATE_HYBRID | UPDATE_FULL))) {
@@ -943,6 +934,13 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		}
 	}
 
+	ret = update_catalog(conf, update);
+	if (ret != KNOT_EOK) {
+		log_zone_error(update->zone->name, "failed to process catalog zone (%s)", knot_strerror(ret));
+		discard_adds_tree(update);
+		return ret;
+	}
+
 	ret = commit_journal(conf, update);
 	if (ret != KNOT_EOK) {
 		discard_adds_tree(update);
@@ -963,14 +961,6 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	zone_contents_t *old_contents;
 	old_contents = zone_switch_contents(update->zone, update->new_cont);
 
-	ret = update_catalog(conf, update);
-	if (ret == KNOT_EZONEINVAL) {
-		log_zone_warning(update->zone->name, "invalid catalog zone version");
-	} else if (ret != KNOT_EOK) {
-		log_zone_warning(update->zone->name, "catalog zone not fully populated (%s)",
-		                 knot_strerror(ret));
-	}
-
 	if (update->flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)) {
 		changeset_clear(&update->change);
 		changeset_clear(&update->extra_ch);
