@@ -102,10 +102,14 @@ static int digest_node(zone_node_t *node, void *ctx)
 int zone_contents_digest(const zone_contents_t *contents, int algorithm,
                          uint8_t **out_digest, size_t *out_size)
 {
-	if (contents == NULL || out_digest == NULL || out_size == NULL) {
+	if (out_digest == NULL || out_size == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	if (contents == NULL) {
+		return KNOT_EEMPTYZONE;
+	}
+
 	contents_digest_ctx_t ctx = {
 		.buf_size = DIGEST_BUF_MIN,
 		.buf = malloc(DIGEST_BUF_MIN),
@@ -169,6 +173,24 @@ static int verify_zonemd(const knot_rdata_t *zonemd, const zone_contents_t *cont
 	return ret;
 }
 
+bool zone_contents_digest_exists(const zone_contents_t *contents, uint8_t alg, bool no_verify)
+{
+	if (alg == 0) {
+		return true;
+	}
+
+	knot_rdataset_t *zonemd = node_rdataset(contents->apex, KNOT_RRTYPE_ZONEMD);
+	if (zonemd == NULL || zonemd->count != 1 || knot_zonemd_algorithm(zonemd->rdata) != alg) {
+		return false;
+	}
+
+	if (no_verify) {
+		return true;
+	}
+
+	return verify_zonemd(zonemd->rdata, contents) == KNOT_EOK;
+}
+
 static bool check_duplicate_schalg(const knot_rdataset_t *zonemd, int check_upto,
                                    uint8_t scheme, uint8_t alg)
 {
@@ -187,7 +209,7 @@ static bool check_duplicate_schalg(const knot_rdataset_t *zonemd, int check_upto
 int zone_contents_digest_verify(const zone_contents_t *contents)
 {
 	if (contents == NULL) {
-		return KNOT_EINVAL;
+		return KNOT_EEMPTYZONE;
 	}
 
 	knot_rdataset_t *zonemd = node_rdataset(contents->apex, KNOT_RRTYPE_ZONEMD);
