@@ -453,6 +453,7 @@ void zone_timers_sanitize(conf_t *conf, zone_t *zone)
 		// invalidate if we don't have a master
 		zone->timers.last_refresh = 0;
 		zone->timers.next_refresh = 0;
+		zone->timers.last_refresh_ok = false;
 	}
 }
 
@@ -618,6 +619,7 @@ int zone_get_lastsigned_serial(zone_t *zone, uint32_t *serial)
 int slave_zone_serial(zone_t *zone, conf_t *conf, uint32_t *serial)
 {
 	int ret = KNOT_EOK;
+	assert(zone->contents != NULL);
 	*serial = zone_contents_serial(zone->contents);
 
 	conf_val_t val = conf_zone_get(conf, C_DNSSEC_SIGNING, zone->name);
