@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <signal.h>
 #include <sys/stat.h>
 #include <urcu.h>
 
@@ -29,10 +30,6 @@
 #include <cap-ng.h>
 #endif
 
-#ifdef ENABLE_SYSTEMD
-#include <systemd/sd-daemon.h>
-#endif
-
 #include "libdnssec/crypto.h"
 #include "libknot/libknot.h"
 #include "contrib/strtonum.h"
@@ -43,6 +40,7 @@
 #include "knot/common/log.h"
 #include "knot/common/process.h"
 #include "knot/common/stats.h"
+#include "knot/common/systemd.h"
 #include "knot/server/server.h"
 #include "knot/server/tcp-handler.h"
 
@@ -53,14 +51,6 @@ static volatile bool sig_req_stop = false;
 static volatile bool sig_req_reload = false;
 static volatile bool sig_req_zones_reload = false;
 
-/* \brief Signal started state to the init system. */
-static void init_signal_started(void)
-{
-#ifdef ENABLE_SYSTEMD
-	sd_notify(0, "READY=1");
-#endif
-}
-
 static int make_daemon(int nochdir, int noclose)
 {
 	int ret;
@@ -258,20 +248,23 @@ static void event_loop(server_t *server, const char *socket)
 
 	enable_signals();
 
+	/* Notify systemd about successful start. */
+	systemd_ready_notify();
+
 	/* Run event loop. */
 	for (;;) {
 		/* Interrupts. */
-		if (sig_req_stop) {
-			break;
-		}
-		if (sig_req_reload) {
+		if (sig_req_reload && !sig_req_stop) {
 			sig_req_reload = false;
 			server_reload(server);
 		}
-		if (sig_req_zones_reload) {
+		if (sig_req_zones_reload && !sig_req_stop) {
 			sig_req_zones_reload = false;
 			server_update_zones(conf(), server);
 		}
+		if (sig_req_stop) {
+			break;
+		}
 
 		// Update control timeout.
 		knot_ctl_set_timeout(ctl, conf()->cache.ctl_timeout);
@@ -526,19 +519,16 @@ int main(int argc, char **argv)
 	                      &conf()->query_plan);
 
 	/* Check and create PID file. */
-	long pid = (long)getpid();
-	if (daemonize) {
-		char *pidfile = pid_check_and_create();
-		if (pidfile == NULL) {
-			server_wait(&server);
-			server_deinit(&server);
-			conf_free(conf());
-			log_close();
-			return EXIT_FAILURE;
-		}
+	unsigned long pid = pid_check_and_create();
+	if (pid == 0) {
+		server_wait(&server);
+		server_deinit(&server);
+		conf_free(conf());
+		log_close();
+		return EXIT_FAILURE;
+	}
 
-		log_info("PID stored in '%s'", pidfile);
-		free(pidfile);
+	if (daemonize) {
 		if (chdir(daemon_root) != 0) {
 			log_warning("failed to change working directory to %s",
 			            daemon_root);
@@ -578,10 +568,9 @@ int main(int argc, char **argv)
 	}
 
 	if (daemonize) {
-		log_info("server started as a daemon, PID %ld", pid);
+		log_info("server started as a daemon, PID %lu", pid);
 	} else {
-		log_info("server started in the foreground, PID %ld", pid);
-		init_signal_started();
+		log_info("server started in the foreground, PID %lu", pid);
 	}
 
 	/* Start the event loop. */
